part of tecfy_corev2_client_package;

class Entity {
  /// arabic name
  late String nameAr;

  /// english name
  late String nameEn;
  late String dbName;
  late List<String> mainFilterPropertyDbNames;
  late List<Property> properties;
  late Map<String, Property> propertiesMap;
  late List<Property> gridProperties;
  late List<Property> mainFilterProperties;
  late List<Property> editProperties;
  late List<Property> addProperties;
  late List<EntityAction> actions;
  late List<ViewTab> tabs;
  late Permission permission;
  late bool allowPermisison;
  String get name => LocalConfigService.isArabic ? nameAr : nameEn;

  bool hasPhoto = false;
  String? photoPropName;
  late Property displayFieldAr;
  late Property displayFieldEn;
  Property get displayField =>
      LocalConfigService.isArabic ? displayFieldAr : displayFieldEn;

  bool isLocal = false;
  bool hasBarcode = false;
  late EntityTypes entityType;

  Entity.fromMap(Map<String, dynamic> row) {
    isLocal = row['isLocal'] ?? false;
    allowPermisison = row['allowPermisison'] ?? true;
    nameAr = HelperModel.getName(row, 'name', 'ar');
    nameEn = HelperModel.getName(row, 'name', 'en');
    dbName = row['dbName'] == null ? '' : lowerFirst(row['dbName']);
    mainFilterPropertyDbNames = row['mainFilterPropertyDbNames'] == null
        ? []
        : row['mainFilterPropertyDbNames']
            .map<String>((e) => e.toString())
            .toList();

    if (row['entityType'] != null)
      entityType = EntityTypes.values[row['entityType']];
    else
      entityType = EntityTypes.Data;

    if (row['permission'] != null) {
      permission = Permission.fromMap(row['permission']);
    } else {
      permission = Permission();
    }
    // print('============================================================');
    // print(row['tabs']);
    if (row['tabs'] != null && row['tabs'].length > 0) {
      tabs = row['tabs'].map<ViewTab>((e) => ViewTab.fromJson(e)).toList();
    } else {
      tabs = [
        ViewTab.fromJson({'dbName': null, nameAr: 'a', nameEn: ''})
      ];
    }

    if (row['properties'] != null) {
      properties = row['properties']
          .map<Property>((p) => Property.getProperty(p)!)
          .toList();

      hasPhoto = properties.any((d) =>
          d.propertyType == PropertyTypes.File &&
          (d as PropertyFile).isImage == true);
      if (hasPhoto)
        photoPropName =
            '${properties.firstWhere((d) => d.propertyType == PropertyTypes.File && (d as PropertyFile).isImage == true).dbName}Id';

      gridProperties = properties
          .where((p) =>
              p.isGrid == true &&
              (p.language == Language.notSelected ||
                  p.language == LocalConfigService.language))
          .toList();
      //intializing propertiesMap
      propertiesMap = {};
      for (var p in properties) {
        propertiesMap[p.dbName] = p;
      }

      editProperties = properties.where((p) => p.isEdit == true).toList();
      addProperties = properties.where((p) => p.isAdd == true).toList();

      if (gridProperties.length == 0) gridProperties = properties;
      editProperties.sort((a, b) => a.editOrder - b.editOrder);
      addProperties.sort((a, b) => a.editOrder - b.editOrder);
      gridProperties.sort((a, b) => a.gridOrder - b.gridOrder);
      mainFilterProperties = properties
          .where((p) => mainFilterPropertyDbNames.contains(p.dbName))
          .toList();
      // gridProperties = gridProperties
      //     .where((p) => !mainFilterPropertyDbNames.contains(p.dbName))
      //     .toList();
    } else {
      properties = [];
      gridProperties = [];
      editProperties = [];
      addProperties = [];
    }

    if (properties.length > 0) {
      var displayFs = properties.where((e) => e.isDisplayField).toList();
      if (displayFs.length == 0) {
        displayFieldEn = properties.first;
        displayFieldAr = properties.first;
      } else if (displayFs.length == 1) {
        displayFieldEn = displayFs.first;
        displayFieldAr = displayFs.first;
      } else {
        displayFieldAr = displayFs.any((e) => e.language == Language.arabic)
            ? displayFs.firstWhere((e) => e.language == Language.arabic)
            : displayFs.first;
        displayFieldEn = displayFs.any((e) => e.language == Language.english)
            ? displayFs.firstWhere((e) => e.language == Language.english)
            : displayFs.last;
      }
    }
    if (row['actions'] != null) {
      actions = row['actions']
          .map<EntityAction>((p) => EntityAction.fromMap(p))
          .toList();
    } else
      actions = [];

    // if (entityType == EntityTypes.ViewOnly ||
    //     entityType == EntityTypes.Lookup) {
    //   permission.canAdd = false;
    //   permission.canUpdate = false;
    //   permission.canDelete = false;
    // }

    hasBarcode = properties.any((e) =>
        e.propertyType == PropertyTypes.Text &&
        ((e as PropertyText).isBarcode) == true &&
        (e.isSearchable) == true);
  }

  String lowerFirst(String? s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  Future<bool> setDefaultValues(Map<String, dynamic> data) async {
    bool updated = false;
    for (Property pro in properties) {
      if (await pro.getDefaultValue(data)) updated = true;
    }
    return updated;
  }

  bool? _isTree;
  bool get isTree {
    if (_isTree == null)
      _isTree = properties.any((d) => d.dbName == 'parentId');
    return _isTree ?? false;
  }

  List<PropertyNumber>? _calcProps;
  List<PropertyNumber> get calcProps {
    if (_calcProps == null)
      _calcProps = properties
          .where((p) =>
              p.propertyType == PropertyTypes.Number &&
              (p as PropertyNumber).calcFormula!.isNotEmpty)
          .toList()
          .cast<PropertyNumber>();
    return _calcProps ?? [];
  }

  Map<String, bool> updatedProps = {};
}

enum EntityTypes {
  Lookup,
  Data,
  MenuItem,
  ViewOnly,
  SubEntity,
  SystemGanarated,
  ServiceSideManagement
}

class ViewTab {
  String? dbName;

  /// arabic name
  late String nameAr;

  /// english name
  late String nameEn;
  late String descriptionAr;
  late String descriptionEn;
  String? viewFormula;
  late int editOrder;

  ViewTab.fromJson(Map<String, dynamic> row) {
    dbName = row['dbName'];
    nameAr = HelperModel.getName(row, 'name', 'ar');
    nameEn = HelperModel.getName(row, 'name', 'en');
    descriptionAr = HelperModel.getName(row, 'description', 'ar');
    descriptionEn = HelperModel.getName(row, 'description', 'en');
    editOrder = row['editOrder'] ?? 0;
    viewFormula = row['viewFormula'];
  }

  String lowerFirst(String? s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }
}
