part of tecfy_corev2_client_package;

class FileProperty extends StatefulWidget {
  final Map<String, dynamic>? fileData;

  /// label string
  final String? label;
  final String? description;

  /// is required
  final bool isRequired;

  /// field id
  final String? fileId;

  /// field id
  final int maxSize;

  // final int maxHeight;
  final double? aspectRatioMax;
  final double? aspectRatioMin;
  final bool isImage;
  final List<String>? fileExtensions;
  final bool readOnly;

  /// on changed
  final void Function(dynamic)? onChanged;

  FileProperty(
      {this.fileData,
      this.label,
      this.description,
      this.maxSize = 200,
      // this.maxHeight = 200,
      this.isImage = false,
      this.fileExtensions,
      this.aspectRatioMax,
      this.aspectRatioMin,
      this.onChanged,
      this.readOnly = false,
      this.fileId,
      this.isRequired = false}); //: assert(fileId != null );
  @override
  _FilePropertyState createState() => _FilePropertyState();
}

class _FilePropertyState extends State<FileProperty> {
  dynamic _selectedFile;
  String? _fileName;

  _onChoosePhoto() async {
    if (widget.isImage) {
      var maxWidth = widget.maxSize;
      // var maxHeight = widget.maxHeight;
      if (maxWidth == 0) maxWidth = 200;
      // if (maxHeight == 0) maxHeight = 200;
      _selectedFile = await ImagePicker().pickImage(
          source: ImageSource.gallery,
          maxWidth: widget.maxSize.toDouble(),
          imageQuality: 100);

      if (_selectedFile != null) {
        AppNavigator.showLoading(context);
        //print('File Length : ${photo.data.length}');
        // show loading and upload the photo
        Uint8List imageData = await _selectedFile!.readAsBytes();
        var decodedImage = await decodeImageFromList(imageData);
        print(decodedImage.width);
        print(decodedImage.height);
        print('-----------------------');

        if ((widget.aspectRatioMin == null && widget.aspectRatioMax == null) ||
            (((decodedImage.width / decodedImage.height) >=
                    widget.aspectRatioMin! &&
                (decodedImage.width / decodedImage.height) <=
                    widget.aspectRatioMax!))) {
          print('Aspect NULL');
          var result = await TecfyBasicApp.apiService.httpPostFile(
              'general/storage/file/fullUrl/admin',
              (await _selectedFile!.readAsBytes()),
              path.basename(_selectedFile!.name));
          Navigator.pop(context);
          if (result.success) {
            widget.onChanged?.call(result.data);
            AppNavigator.showMessage(
                context, 'تم رفع الصورة بنجاح', MessageType.success);
          } else {
            print('XXXXXXXXXXXXXXXXXXXX Image Upload Error: ${result.message}');
            AppNavigator.showMessage(
                context, 'خطأ اثناء رفع الصوره', MessageType.error,
                message: result.message);
            _selectedFile = null;
          }
        } else {
          AppNavigator.pop(context);
          AppNavigator.showMessage(
              context,
              'يجب أن تكون أقل نسبة لأبعاد '
              'الصورة : ${widget.aspectRatioMin!.toStringAsFixed(2)}\n  و أقصي '
              'نسبة : ${widget.aspectRatioMax!.toStringAsFixed(2)}',
              MessageType.error);
        }

        // if ((decodedImage.width / decodedImage.height) == widget.aspectRatio) {
        //   var result = await TecfyBasicApp.apiService.httpPostFile(
        //       'general/storage/file/fullUrl/admin',
        //       (await _photo!.readAsBytes()),
        //       path.basename(_photo!.name));
        //   Navigator.pop(context);
        //   if (result.success) {
        //     widget.onChanged?.call(result.data);
        //     AppNavigator.showMessage(
        //         context, 'تم رفع الصورة بنجاح', MessageType.success);
        //   } else {
        //     print('XXXXXXXXXXXXXXXXXXXX Image Upload Error: ${result.message}');
        //     AppNavigator.showMessage(
        //         context, 'خطأ اثناء رفع الصوره', MessageType.error,
        //         message: result.message);
        //     _photo = null;
        //   }
        // } else {
        //   _photo = null;
        //   Navigator.pop(context);
        //   AppNavigator.showMessage(
        //       context,
        //       'يجب أن تكون أبعاد الصورة مناسبة '
        //       'لهذه الأبعاد \n'
        //       'العرض : ${widget.maxSize} '
        //       'الطول : ${widget.maxSize / widget.aspectRatio!}',
        //       MessageType.error);
        // }
        //print(result.toString());
      }
    } else {
      print('<<<<<<<<<<<<<<<<  Not Image  >>>>>>>>>>>>>');
      FilePickerResult? result = await FilePicker.platform.pickFiles(
          allowedExtensions: widget.fileExtensions, //['xlsx'],
          type: FileType.custom);

      if (result != null) {
        Uint8List? bytes;
        AppNavigator.showLoading(context);
        if (kIsWeb) {
          bytes = result.files.single.bytes;
        } else {
          File file = File(result.files.single.path!);
          _selectedFile = file;
          bytes = (await (_selectedFile! as File).readAsBytes());
        }
        if (bytes == null) return;
        var uploadResponse = await TecfyBasicApp.apiService.httpPostFile(
            'general/storage/file/fullUrl/admin',
            bytes,
            result.names.first!,
            {},
            {},
            false);
        Navigator.pop(context);
        if (uploadResponse.success) {
          _fileName = result.names.first!;
          widget.onChanged?.call(uploadResponse.data);
          AppNavigator.showMessage(
              context, 'تم رفع الملف بنجاح', MessageType.success);
        } else {
          print(
              'XXXXXXXXXXXXXXXXXXXX File Upload Error: ${uploadResponse.message}');
          AppNavigator.showMessage(
              context, 'خطأ اثناء رفع الملف', MessageType.error,
              message: uploadResponse.message);
          _selectedFile = null;
        }
      } else {
        // User canceled the picker
      }
    }
    setState(() {});
  }

  _onViewPhoto() async {
    print(widget.fileData);
    if (widget.fileData != null && widget.fileData!['_id'] != null) {
      WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
        AppNavigator.showAppDialogEx(
            context,
            () => PinchZoom(
                child: Image.network(widget.fileData!['_id']),
                zoomEnabled: true),
            size: DialogBoxSizes.large);
      });
    }
  }

  _onChoosePhotoFromFileManager() async {
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) async {
      dynamic result = await AppNavigator.showAppDialogEx(context,
          () => FileManagerPage(isPopup: true, isMultipleChoices: false),
          size: DialogBoxSizes.large);
      if (result != null && result is FileModel) {
        widget.fileData?['_id'] = result.url;
      } else {
        print(result);
      }
      setState(() {
        widget.onChanged?.call(result.url);
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: EdgeInsets.all(10),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          //read only case :
          widget.readOnly
              ? Tooltip(
                  message: widget.description ?? widget.label ?? '',
                  child: widget.isImage
                      ? widget.fileData!['_id'] != null || widget.fileId != null
                          ? Stack(
                              alignment: Alignment.topLeft,
                              children: [
                                CircleAvatar(
                                    backgroundImage: ImageService.imageProvider(
                                        widget.fileData != null &&
                                                widget.fileData!['_id'] != null
                                            ? widget.fileData!['_id']
                                            : widget.fileId),
                                    radius: 55.0),
                                Align(
                                  alignment: Alignment.topLeft,
                                  child: IconButton(
                                      tooltip: AppLocale.translate('Download'),
                                      onPressed: () async {
                                        if (widget.fileData != null &&
                                            widget.fileData!['_id'] != null) {
                                          if (await canLaunchUrl(Uri.parse(
                                              widget.fileData!['_id']))) {
                                            await launchUrl(Uri.parse(
                                                widget.fileData!['_id']));
                                          }
                                        } else {
                                          if (await canLaunchUrl(
                                              Uri.parse(widget.fileId!))) {
                                            await launchUrl(
                                                Uri.parse(widget.fileId!));
                                          }
                                        }
                                      },
                                      icon: Icon(Icons.download)),
                                )
                              ],
                            )
                          : Icon(Icons.photo_size_select_actual, size: 30)
                      : TextFormWidget(
                          description: widget.description,
                          value: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              Text(AppLocale.translate('File')),
                              IconButton(
                                  tooltip: AppLocale.translate('Download'),
                                  onPressed: () async {
                                    if (widget.fileData != null &&
                                        widget.fileData!['_id'] != null) {
                                      if (await canLaunchUrl(
                                          Uri.parse(widget.fileData!['_id']))) {
                                        await launchUrl(
                                            Uri.parse(widget.fileData!['_id']));
                                      }
                                    } else {
                                      if (await canLaunchUrl(
                                          Uri.parse(widget.fileId!))) {
                                        await launchUrl(
                                            Uri.parse(widget.fileId!));
                                      }
                                    }
                                  },
                                  icon: Icon(Icons.download))
                            ],
                          ),
                        ))
              //editable case :
              : DropdownButtonHideUnderline(
                  child: DropdownButton2(
                    isExpanded: true,
                    dropdownStyleData: DropdownStyleData(
                      width: 250,
                    ),
                    customButton: Tooltip(
                      message: widget.description ?? widget.label ?? '',
                      child: _selectedFile != null
                          ? widget.isImage
                              ? CircleAvatar(
                                  backgroundImage:
                                      Image.file(File(_selectedFile!.path))
                                          .image,
                                  radius: 55.0,
                                )
                              : Wrap(
                                  children: <Widget>[
                                    if (_fileName != null)
                                      Container(
                                        width: 150,
                                        child: Center(child: Text(_fileName!)),
                                        decoration: BoxDecoration(
                                            border: Border.all(
                                                width: 2,
                                                color: Colors.grey
                                                    .withOpacity(0.6)),
                                            borderRadius:
                                                BorderRadius.circular(3)),
                                      ),
                                    SizedBox(width: 20),
                                    ElevatedButton(
                                        onPressed: _onChoosePhoto,
                                        child: Text('Browse')),
                                  ],
                                )
                          : widget.fileData!['_id'] != null ||
                                  widget.fileId != null
                              ? widget.isImage
                                  ? CircleAvatar(
                                      backgroundImage:
                                          ImageService.imageProvider(
                                              widget.fileData != null &&
                                                      widget.fileData!['_id'] !=
                                                          null
                                                  ? widget.fileData!['_id']
                                                  : widget.fileId),
                                      radius: 55.0)
                                  : Wrap(
                                      crossAxisAlignment:
                                          WrapCrossAlignment.center,
                                      children: <Widget>[
                                        Container(
                                          child: Text(
                                              _fileName ??
                                                  'Click Browse to select your '
                                                      'file...',
                                              overflow: TextOverflow.ellipsis),
                                          width: 120,
                                          decoration: BoxDecoration(
                                              color: Colors.transparent,
                                              border: Border.all(
                                                  width: 0.5,
                                                  color: Colors.black),
                                              borderRadius:
                                                  BorderRadius.circular(3)),
                                        ),
                                        SizedBox(width: 4),
                                        ElevatedButton(
                                            onPressed: _onChoosePhoto,
                                            child: Text('Browse'),
                                            style: ElevatedButton.styleFrom(
                                                backgroundColor: Colors.grey)),
                                        IconButton(
                                            tooltip:
                                                AppLocale.translate('Download'),
                                            onPressed: () async {
                                              if (widget.fileData != null &&
                                                  widget.fileData!['_id'] !=
                                                      null) {
                                                if (await canLaunchUrl(
                                                    Uri.parse(widget
                                                        .fileData!['_id']))) {
                                                  await launchUrl(Uri.parse(
                                                      widget.fileData!['_id']));
                                                }
                                              } else {
                                                if (await canLaunchUrl(
                                                    Uri.parse(
                                                        widget.fileId!))) {
                                                  await launchUrl(Uri.parse(
                                                      widget.fileId!));
                                                }
                                              }
                                            },
                                            icon: Icon(Icons.download))
                                      ],
                                    )
                              :
                              // Image.asset('assets/images/profile_pic.png',height: 110.0)
                              widget.isImage
                                  ? Column(
                                      children: <Widget>[
                                        Icon(Icons.photo_size_select_actual,
                                            size: 30),
                                        TecfyText(
                                            '${widget.label ?? 'Select Image'}${widget.isRequired ? '*' : ''}')
                                      ],
                                    )
                                  : Wrap(
                                      crossAxisAlignment:
                                          WrapCrossAlignment.center,
                                      children: <Widget>[
                                        Container(
                                          child: Text(
                                              _fileName ??
                                                  'Click Browse to select your '
                                                      'file...',
                                              overflow: TextOverflow.ellipsis),
                                          width: 150,
                                          decoration: BoxDecoration(
                                              color: Colors.transparent,
                                              border: Border.all(
                                                  width: 0.5,
                                                  color: Colors.black),
                                              borderRadius:
                                                  BorderRadius.circular(3)),
                                        ),
                                        SizedBox(width: 30),
                                        ElevatedButton(
                                            onPressed: _onChoosePhoto,
                                            child: Text('Browse'),
                                            style: ElevatedButton.styleFrom(
                                                backgroundColor: Colors.grey)),
                                      ],
                                    ),
                    ),
                    items: widget.isImage
                        ? [
                            DropdownMenuItem(
                                value: 'upload',
                                onTap: _onChoosePhoto,
                                child: Text('Upload')),
                            DropdownMenuItem(
                                value: 'view',
                                onTap: _onViewPhoto,
                                child: Text('View')),
                            DropdownMenuItem(
                              value: 'upload',
                              onTap: _onChoosePhotoFromFileManager,
                              child: Text('Select from File Manager',
                                  overflow: TextOverflow.ellipsis, maxLines: 1),
                            ),
                          ]
                        : [
                            DropdownMenuItem(
                                value: 'upload',
                                onTap: _onChoosePhoto,
                                child: Text('Upload')),
                            DropdownMenuItem(
                              value: 'upload',
                              onTap: _onChoosePhotoFromFileManager,
                              child: Text('Select from File Manager',
                                  overflow: TextOverflow.ellipsis, maxLines: 1),
                            ),
                          ],
                    onChanged: (value) {},
                  ),
                ),
        ],
      ),
    );
  }
}
