part of tecfy_corev2_client_package;

class EntityListPage extends StatefulWidget {
  /// the main entity of the page
  final String entity;

  /// the default filter
  final List<EntityFilter>? filters;

  /// the default order property
  final String? orderBy;

  /// if the order property is desc
  final bool orderByDesc;

  /// if true, the user will not be able to add/edit/delete,
  /// only view data in a list
  final bool isListOnly;

  /// if true, the page will show the mobile view even for desktop
  final bool isPopup;

  /// Create a Page show recors from database in a grid,
  /// with option to add/update/delete
  EntityListPage(this.entity,
      {this.filters,
      this.orderBy,
      this.orderByDesc = false,
      this.isListOnly = false,
      this.isPopup = false});
  @override
  _EntityListPageState createState() => _EntityListPageState();
}

class _EntityListPageState
    extends TecfyStateView<EntityListPage, EntityListController> {
  @override
  void initState() {
    if (kDebugMode) print('open list page ${widget.entity}');
    _controller = EntityListController(this, widget.entity,
        filters: widget.filters,
        orderBy: widget.orderBy,
        isPopup: widget.isPopup,
        orderByDesc: widget.orderByDesc);
    super.initState();
  }

  Widget _getMenu() {
    List<PopupMenuEntry<String>> menuItems = [
      PopupMenuItem<String>(
        value: 'filter',
        textStyle: TextStyle(color: ThemeService.textOnMainColor),
        child: Row(
          children: [
            Expanded(child: TecfyText(trans('Filter'))),
            Icon(
              Icons.filter,
              color: ThemeService.textOnMainColor,
              semanticLabel: trans('Filter'),
            ),
          ],
        ),
        enabled: true,
      ),
      PopupMenuItem<String>(
        value: 'merge',
        textStyle: TextStyle(color: ThemeService.textOnMainColor),
        child: Row(
          children: [
            Expanded(child: TecfyText(trans('Merge'))),
            Icon(
              Icons.merge_type,
              color: ThemeService.textOnMainColor,
              semanticLabel: trans('Merge'),
            ),
          ],
        ),
        enabled: true,
      ),
      PopupMenuItem<String>(
        value: 'export',
        textStyle: TextStyle(color: ThemeService.textOnMainColor),
        child: Row(
          children: [
            Expanded(child: TecfyText(trans('Export'))),
            Icon(
              Icons.cloud_download_sharp,
              color: ThemeService.textOnMainColor,
              semanticLabel: trans('Export'),
            ),
          ],
        ),
        enabled: true,
      ),
    ];

    var actions = _controller.entityActionsService?.getPageActions().data ?? [];
    for (var action in actions) {
      menuItems.add(PopupMenuItem<String>(
        value: action.dbName,
        textStyle: TextStyle(color: ThemeService.textOnMainColor),
        child: Row(
          children: [
            Expanded(child: TecfyText(action.name)),
            Icon(
              action.iconData,
              color: ThemeService.textOnMainColor,
              semanticLabel: action.name,
            ),
          ],
        ),
        enabled: true,
      ));
    }
    return PopupMenuButton<String>(
      //onSelected: _controller.choicePopMenuAction,
      onSelected: (cmd) {
        if (cmd == 'filter') {
          _controller.onOpenFilter();
        } else if (cmd == 'merge') {
          _controller.onOpenMerge();
        } else if (cmd == 'export') {
          _controller.onExport();
        } else {
          _controller.onAction(actions.firstWhere((a) => a.dbName == cmd), {});
        }
      },
      color: Color.fromRGBO(73, 77, 89, 1),
      itemBuilder: (BuildContext context) => menuItems,
    );
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return widget.isPopup
        ? LayoutService.bindWidget(context,
            mobileWidget: _getMobileWidget,
            // tabletWidget: _getTabletWidget,
            descktopWidget: _getDesktopWidget)
        : BasePage(context, _controller.entity.name, _controller.entity.dbName,
            isMobile: LayoutService.isMobile(context),
            appBarActions: LayoutService.isMobile(context)
                ? [
                    // IconButton(
                    //     icon: Icon(Icons.filter_list,
                    //         color: _controller.filters!.length > 0
                    //             ? Colors.redAccent
                    //             : ThemeService.mainColor),
                    //     onPressed: _onOpenFilter),
                    _getMenu()
                  ]
                : null,
            floatingActionButtonWidget: LayoutService.isMobile(context) &&
                    !widget.isListOnly &&
                    _controller.entity.permission.canAdd &&
                    _controller.entity.entityType != EntityTypes.ViewOnly &&
                    _controller.entity.entityType != EntityTypes.Lookup
                ? FloatingActionButton(
                    backgroundColor: ThemeService.mainColor,
                    onPressed: () async => _controller.onAdd(),
                    child: Icon(Icons.add))
                : null,
            child: _controller.data == null
                ? Center(child: AppNavigator.getLoadingWidget(70))
                : LayoutService.bindWidget(context,
                    mobileWidget: _getMobileWidget,
                    // tabletWidget: _getTabletWidget,
                    descktopWidget: _getDesktopWidget));
  }

  Map<int, TableColumnWidth> _getColumnWidths() {
    var columns = _controller.entity.gridProperties.toList();

    Map<int, TableColumnWidth> data = {};
    int testCount = 0;
    var isTree = _controller.entity.isTree;

    if (isTree) data[0] = FixedColumnWidth(60);

    for (int i = 0; i < columns.length; i++) {
      switch (columns[i].propertyType) {
        case PropertyTypes.Text:
          data[isTree ? 1 + i : i] = FlexColumnWidth(testCount == 1 ? 2 : 1);
          testCount++;
          break;
        case PropertyTypes.File:
          data[isTree ? 1 + i : i] = FixedColumnWidth(60);
          break;
        case PropertyTypes.Number:
          data[isTree ? 1 + i : i] = FlexColumnWidth(0.5);
          break;
        case PropertyTypes.Boolean:
          data[isTree ? 1 + i : i] = FixedColumnWidth(40);
          break;
        default:
          data[isTree ? 1 + i : i] = FlexColumnWidth(1);
          break;
      }
    }

    data[isTree ? 1 + columns.length : columns.length] = FixedColumnWidth(40);
    data[isTree ? 2 + columns.length : 1 + columns.length] =
        FixedColumnWidth(40);

    if (_controller.hasActions(null)) {
      if (!widget.isListOnly &&
          _controller.entity.permission.canDelete &&
          _controller.entity.entityType != EntityTypes.ViewOnly &&
          _controller.entity.entityType != EntityTypes.Lookup)
        data[isTree ? 3 + columns.length : 2 + columns.length] =
            FixedColumnWidth(
                _controller.getActions(null).length > 0 ? 40.0 : 0);
      else
        data[isTree ? 2 + columns.length : 1 + columns.length] =
            FixedColumnWidth(
                _controller.getActions(null).length > 0 ? 40.0 : 0);
    }
    return data;
  }

  Widget _getDesktopWidget() {
    List<Property> props = _controller.entity.mainFilterProperties.map((e) {
      if (e is PropertyDropdown) {
        return Property.getProperty({
          'entityName': e.entityName,
          'filterValueField': e.filterValueField,
          ...e.toJson(),
        })!;
      } else {
        return Property.getProperty(e.toJson())!;
      }
    }).toList();

    props.forEach((element) {
      element.isRequired = false;
      element.readOnly = false;
    });
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        if (props.isNotEmpty)
          Row(
              children: props
                  .map((prop) => Expanded(
                          child: Container(
                        padding: EdgeInsets.symmetric(horizontal: 10),
                        child: PorpertyComponent(
                            property: prop,
                            entity: _controller.entity,
                            data: _controller.getFiltersData(),
                            onChange: (v) =>
                                _controller.mainFilterChanged(prop, v)),
                      )))
                  .toList()),
        Row(children: <Widget>[
          Expanded(
              child: SearchBox(
            searchTextController: _controller.searchController,
            placeholder: trans('Search'),
            onChange: (v) => _controller.globalSearchText(),
            onSubmit: (v) =>
                _controller.globalSearchTextServer(clearPagination: true),
          )),
          SizedBox(width: 10),
          !widget.isListOnly &&
                  _controller.entity.permission.canAdd &&
                  _controller.entity.entityType != EntityTypes.ViewOnly &&
                  _controller.entity.entityType != EntityTypes.Lookup
              ? AppButtonIconText(
                  onPressed: () async => _controller.onAdd(),
                  label: trans('Add'),
                  height: 35,
                  icon: Icon(Icons.add, color: ThemeService.textOnMainColor),
                  color: ThemeService.secondryColor)
              : Container(),
          SizedBox(width: 10),
          AppButtonIconText(
              onPressed: _controller.onRefresh,
              label: trans('Refresh'),
              height: 35,
              icon: Icon(Icons.refresh, color: ThemeService.textOnMainColor),
              color: ThemeService.subColor),
          //SizedBox(width: 10),
          _getMenu(),
        ]),
        Expanded(
            child: Scrollbar(
                // thumbVisibility: true,
                // trackVisibility: true,
                controller: _controller.scrollController,
                child: SingleChildScrollView(
                  controller: _controller.scrollController,
                  child: Container(
                      margin: EdgeInsets.symmetric(horizontal: 30, vertical: 5),
                      child: AnimationLimiter(
                        //key:UniqueKey(),
                        child: Table(
                          columnWidths: _getColumnWidths(),
                          children: _getTableRows(),
                        ),
                      )),
                ))),
        _controller.hasPageing
            ? SizedBox(
                height: 50,
                child: Row(
                  children: [
                    Expanded(
                      child: ListView.builder(
                        itemCount: (_controller.pageIndexMax + 2),
                        scrollDirection: Axis.horizontal,
                        itemBuilder: (_context, pageIndex) => pageIndex ==
                                _controller.pageIndexMax + 1
                            ? (_controller.hasNextPage
                                ? TextButton(
                                    child: TecfyText(AppLocale.translate('Next '
                                        'Page')),
                                    onPressed: _controller.gotoNextPageIndex)
                                : Container())
                            : IconButton(
                                padding: EdgeInsets.all(0),
                                icon: TecfyText(
                                  (pageIndex + 1).toString(),
                                  style: TextStyle(
                                      color: pageIndex == _controller.pageIndex
                                          ? ThemeService.mainColor
                                          : ThemeService.secondryColor),
                                ),
                                onPressed: () =>
                                    _controller.gotoPageIndex(pageIndex),
                              ),
                      ),
                    ),
                    SizedBox(width: 10),
                    TextButton(
                        child: TecfyText(
                            '${(_controller.filterdData ?? _controller.data)?.length ?? '?'}/${_controller.totalCount ?? '?'}'),
                        onPressed: _controller.getTotalCount),
                    SizedBox(width: 10),
                  ],
                ),
              )
            : Container()
      ],
    );
  }

  // Widget _getTabletWidget() {
  //   return Column(
  //     crossAxisAlignment: CrossAxisAlignment.stretch,
  //     children: <Widget>[
  //       Row(children: <Widget>[
  //         Expanded(
  //             child: SearchBox(
  //           searchTextController: _controller.searchController,
  //           placeholder: trans('Search'),
  //           onChange: (v) => _controller.globalSearchText(),
  //           onSubmit: (v) => _controller.globalSearchTextServer(),
  //         )),
  //         !widget.isListOnly &&
  //                 _controller.entity.permission.canAdd &&
  //                 _controller.entity.entityType != EntityTypes.ViewOnly &&
  //                 _controller.entity.entityType != EntityTypes.Lookup
  //             ? AppButtonIconText(
  //                 onPressed: () async => _controller.onAdd(),
  //                 label: trans('Add'),
  //                 height: 35,
  //                 icon: Icon(Icons.add, color: ThemeService.textOnMainColor),
  //                 color: ThemeService.secondryColor)
  //             : Container(),
  //         SizedBox(width: 10),
  //         AppButtonIconText(
  //             onPressed: _controller.onRefresh,
  //             label: trans('Refresh'),
  //             height: 35,
  //             icon: Icon(Icons.refresh, color: ThemeService.textOnMainColor),
  //             color: ThemeService.subColor),
  //         _getMenu()
  //       ]),
  //       Expanded(
  //         child: Scrollbar(
  //           controller: _controller.scrollController,
  //           child: SingleChildScrollView(
  //             controller: _controller.scrollController,
  //             child: Container(
  //                 margin: EdgeInsets.symmetric(horizontal: 30, vertical: 5),
  //                 child: AnimationLimiter(
  //                   //key:UniqueKey(),
  //                   child: Table(
  //                     columnWidths: _getColumnWidths(),
  //                     children: _getTableRows(),
  //                   ),
  //                 )),
  //           ),
  //         ),
  //       ),
  //       _controller.hasPageing
  //           ? SizedBox(
  //               height: 50,
  //               child: ListView.builder(
  //                 itemCount: (_controller.pageIndexMax + 2),
  //                 scrollDirection: Axis.horizontal,
  //                 itemBuilder: (_context, pageIndex) => pageIndex ==
  //                         _controller.pageIndexMax + 1
  //                     ? (_controller.hasNextPage
  //                         ? TextButton(
  //                             child: TecfyText(AppLocale.translate('Next '
  //                                 'Page')),
  //                             onPressed: _controller.gotoMextPageIndex)
  //                         : Container())
  //                     : IconButton(
  //                         padding: EdgeInsets.all(0),
  //                         icon: TecfyText(
  //                           (pageIndex + 1).toString(),
  //                           style: TextStyle(
  //                               color: pageIndex == _controller.pageIndex
  //                                   ? ThemeService.mainColor
  //                                   : ThemeService.secondryColor),
  //                         ),
  //                         onPressed: () => _controller.gotoPageIndex(pageIndex),
  //                       ),
  //               ),
  //             )
  //           : Container()
  //     ],
  //   );
  // }

  List<TableRow> _addChildren(items, List<TableRow> rows) {
    if (items != null)
      for (var item in items) {
        rows.add(_getTableRow(item, rowIndex++));
        // if (item['_expanded'] == true &&
        //     item['_children'] != null &&
        //     item['_children'].length > 0) {
        //   rows = addChildren(item['_children'], rows);
        // }
      }
    return rows;
  }

  int rowIndex = 0;
  List<TableRow> _getTableRows() {
    List<TableRow> rows = [];
    rows.add(_getHeaderRow());
    rowIndex = 1;
    return _addChildren(_controller.filterdData, rows);
  }

  TableRow _getTableRow(d, rowIndex) {
    var props = _controller.entity.gridProperties
        .map<Widget>((p) => _getCell(p, d))
        .toList();
    if (!widget.isListOnly) {
      props.add(_editCell(d));
    }
    if (!widget.isListOnly &&
        _controller.entity.permission.canDelete &&
        _controller.entity.entityType != EntityTypes.ViewOnly &&
        _controller.entity.entityType != EntityTypes.Lookup)
      props.add(_deleteCell(d));

    if (_controller.entity.isTree) props.insert(0, _treeCell(d));

    if (_controller.hasActions(null)) {
      var actions = _controller.getActions(d);
      if (actions.length == 1) {
        props.add(AppIconButton(
          icon: actions.first.icon, // ?? Icon(Icons.local_activity_outlined),

          tooltip: actions.first.name,
          onPressed: () => _controller.onAction(actions.first, d),
        ));
      } else if (actions.length > 1) {
        props.add(PopupMenuButton<String>(
            onSelected: (cmd) {
              if (cmd == 'filter') {
                _controller.onOpenFilter();
              } else if (cmd == 'merge') {
                _controller.onOpenMerge();
              } else if (cmd == 'export') {
                _controller.onExport();
              }
            },
            color: Colors.white,
            icon: Icon(Icons.more_horiz, color: Colors.black),
            itemBuilder: (BuildContext context) => actions
                .map<PopupMenuEntry<String>>(
                  (action) => PopupMenuItem<String>(
                    value: action.dbName,
                    child: AppButtonIconText(
                      height: 30,
                      isFullRow: true,
                      color: Colors.transparent,
                      icon:
                          Icon(action.iconData, color: ThemeService.mainColor),
                      label: action.name,
                      onPressed: () => _controller.onAction(action, d),
                    ),
                    enabled: true,
                  ),
                )
                .toList()));
      } else {
        props.add(Container());
      }
    }

    return TableRow(
      decoration: BoxDecoration(
        color: d['_tree_level'] == null
            ? rowIndex % 2 == 0
                ? ThemeService.backgroundSecondColor
                : ThemeService.backgroundColor
            : ThemeService.mainColor.withOpacity(d['_tree_level'] * 0.1),
      ),
      children: AnimationConfiguration.toStaggeredList(
        duration: const Duration(milliseconds: 375),
        childAnimationBuilder: (widget) => SlideAnimation(
            horizontalOffset: 50.0,
            child: SlideAnimation(
              verticalOffset: 50.0,
              child: FadeInAnimation(
                child: widget,
              ),
            )),
        children: props,
      ),
    );
  }

  TableRow _getHeaderRow() {
    var props = _controller.entity.gridProperties
        .map<TableCell>((p) => _getHeader(p))
        .toList();
    if (!widget.isListOnly) {
      props.add(_getHeaderCell(
          _controller.entity.permission.canUpdate &&
                  _controller.entity.entityType != EntityTypes.ViewOnly &&
                  _controller.entity.entityType != EntityTypes.Lookup
              ? trans('Edit')
              : trans('Details'),
          '',
          false,
          null,
          true));
    }

    if (!widget.isListOnly &&
        _controller.entity.permission.canDelete &&
        _controller.entity.entityType != EntityTypes.ViewOnly &&
        _controller.entity.entityType != EntityTypes.Lookup)
      props.add(
          _getHeaderCell(AppLocale.translate('Delete'), '', false, null, true));
    if (_controller.hasActions(null))
      props.add(
          _getHeaderCell(AppLocale.translate('Action'), '', false, null, true));
    if (_controller.entity.isTree)
      props.insert(0, _getHeaderCell('--', '', false, null, true));
    return TableRow(children: props);
  }

  Widget _editCell(Map<String, dynamic> data) {
    return IconButton(
      tooltip: !widget.isListOnly &&
              _controller.entity.permission.canUpdate &&
              _controller.entity.entityType != EntityTypes.ViewOnly &&
              _controller.entity.entityType != EntityTypes.Lookup
          ? trans('Edit')
          : trans('Details'),
      padding: EdgeInsets.all(0),
      icon: Icon(Icons.note, color: ThemeService.mainColor),
      onPressed: () => !widget.isListOnly &&
              _controller.entity.permission.canUpdate &&
              _controller.entity.entityType != EntityTypes.ViewOnly &&
              _controller.entity.entityType != EntityTypes.Lookup
          ? _controller.onUpdate(data)
          : _controller.onView(data),
    );
  }

  Widget _treeCell(Map<String, dynamic> data) {
    if (data['_children'] != null && data['_children'].length == 0)
      return Container();
    return IconButton(
      tooltip: trans(data['_expanded'] == true ? 'Collapse' : 'Expand'),
      padding: EdgeInsets.only(
          right: LocalConfigService.isArabic
              ? ((data['_tree_level'] as num?) ?? 0).toDouble() * 20.0
              : 0.0,
          left: LocalConfigService.isArabic
              ? 0.0
              : ((data['_tree_level'] ?? 0) as num).toDouble() * 20.0),
      icon: data['_expanded'] == true
          ? Icon(Icons.arrow_drop_down, size: 26)
          : RotatedBox(
              quarterTurns: LocalConfigService.isArabic ? 2 : 4,
              child: Icon(Icons.play_arrow_outlined, size: 20),
            ),

      // Icon(
      //     data['_expanded'] == true
      //         ? Icons.keyboard_arrow_up_outlined
      //         : Icons.keyboard_arrow_down_outlined,
      //     color: ThemeService.mainColor,
      //     size: 22),
      onPressed: () => _controller.expandItem(data),
    );
  }

  Widget _deleteCell(Map<String, dynamic> data) {
    return data['isLocked'] == true ||
            !_controller.entity.permission.canDelete ||
            widget.isListOnly ||
            _controller.entity.entityType == EntityTypes.ViewOnly ||
            _controller.entity.entityType == EntityTypes.Lookup
        ? Container()
        : IconButton(
            tooltip: trans('Delete'),
            padding: EdgeInsets.all(0),
            icon: Icon(Icons.delete, color: ThemeService.dangerColor),
            onPressed: () => _controller.onDelete(data),
          );
  }

  TableCell _getHeader(Property prop) {
    return _getHeaderCell(prop.name, prop.dbName,
        prop.dbName == _controller.orderBy, () => _controller.sortBy(prop));
  }

  TableCell _getHeaderCell(String text, String dbName,
      [bool isSelected = false, onPressed, bool isCenter = false]) {
    return TableCell(
        child: MaterialButton(
            padding: EdgeInsets.all(0),
            child: Align(
                alignment: isCenter
                    ? Alignment.center
                    : LocalConfigService.isArabic
                        ? Alignment.centerRight
                        : Alignment.centerLeft,
                child: Tooltip(
                  message: dbName,
                  child: TecfyText('$text ${isSelected ? '*' : ''}',
                      textAlign: TextAlign.start,
                      style: TextStyle(
                          fontSize: 12,
                          fontWeight:
                              isSelected ? FontWeight.bold : FontWeight.normal,
                          color: ThemeService.secondryColor
                          //color: isSelected ? ThemeService.mainColor : ThemeService.secondryColor
                          )),
                )),
            onPressed: onPressed));
  }

  Widget _getCell(Property prop, data, {totals = false}) {
    //Download Button in File Properties if property is not Image instead of the action of Download
    if (prop.propertyType == PropertyTypes.File) {
      if (prop.isImage != true) {
        if (data[prop.dbName] != null) {
          return Center(
            child: Align(
              alignment: LocalConfigService.isArabic
                  ? Alignment.centerRight
                  : Alignment.centerLeft,
              child: Tooltip(
                message: AppLocale.translate('Download'),
                child: InkWell(
                    onTap: () async {
                      if (data[prop.dbName]!.startsWith('https')) {
                        if (await canLaunchUrl(Uri.parse(data[prop.dbName]!))) {
                          await launchUrl(Uri.parse(data[prop.dbName]!));
                        }
                      }
                    },
                    child: Icon(Icons.download)),
              ),
            ),
          );
        } else {
          return const SizedBox();
        }
      }
    }
    if (prop.isImage) {
      if (data[prop.dbName] == null || data[prop.dbName].toString().isEmpty) {
        return Container();
      }
      return Align(
          alignment: LocalConfigService.isArabic
              ? Alignment.centerRight
              : Alignment.centerLeft,
          child: CircleAvatar(
            radius: 20,
            backgroundImage: ImageService.imageProvider(
                    (MapService.getValue(data, prop.dbName)) is Map
                        ? MapService.getValue(data, prop.dbName)['_id']
                        : MapService.getValue(data, prop.dbName)) ??
                TecfyCoreApp.logo!.image,
          ));
    }
    // var val = data[prop.dbName];
    // if (totals) {
    //   if (val == null || val == 0 || val == '' || val == '0') {
    //     return Container();
    //   }
    // } else {
    //   if (val == null) {
    //     return Container();
    //   }
    // }

    // Widget widgetValue;
    // switch (prop.valueType) {
    //   case ValueTypes.TecfyText:
    //     widgetValue = TecfyText(
    //       val.toString(),
    //       style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 16),
    //     );
    //     break;
    //   case ValueTypes.Number:
    //     //widgetValue =TecfyText(val.toString(),style: totals? ThemeService
    //     .titleStyle :TextStyle(fontSize: 12));
    //     if (prop.propertyType == PropertyTypes.DateTime)
    //       widgetValue = TecfyText(
    //         DateTime.fromMillisecondsSinceEpoch(val, isUtc: true).toString(),
    //         style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 10),
    //       );
    //     else
    //       widgetValue = prop.getDisplayWidget(data);
    //     break;
    //   case ValueTypes.DateTime:
    //     if (prop.propertyType == PropertyTypes.DateTime)
    //       widgetValue = TecfyText(
    //         val.toString().replaceFirst('T', ' ').substring(0, 16),
    //         style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 10),
    //       );
    //     else
    //       widgetValue = TecfyText(
    //         val.toString().length > 10
    //             ? val.toString().substring(0, 10)
    //             : val.toString(),
    //         style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 10),
    //       );
    //     break;
    //   case ValueTypes.Boolean:
    //     widgetValue = Padding(
    //       padding: EdgeInsets.only(top: 7, right: 2, left: 2),
    //       child: Align(
    //         alignment: LocalConfigService.isArabic
    //             ? Alignment.centerRight
    //             : Alignment.centerLeft,
    //         child: prop.getDisplayWidget(data),
    //       ),
    //     ); // TecfyText( val.toString(),style: totals? ThemeService
    //     .titleStyle
    //     :TextStyle(fontSize: 10));
    //     break;
    //   default:
    //     widgetValue = TecfyText('N/A [${prop.valueType}]');
    // }

    Widget widgetValue = Padding(
      padding: EdgeInsets.only(
          top: prop.valueType == ValueTypes.Boolean ? 7 : 0, right: 2, left: 2),
      child: Align(
        alignment: LocalConfigService.isArabic
            ? Alignment.centerRight
            : Alignment.centerLeft,
        child: prop.getDisplayWidget(data),
      ),
    );
    return Padding(
        padding: EdgeInsets.only(
            top: 10.0,
            right: LocalConfigService.isArabic
                ? (((data['_tree_level'] ?? 0) as num).toDouble()) * 10.0
                : 0.0,
            left: LocalConfigService.isArabic
                ? 0.0
                : (((data['_tree_level'] ?? 0) as num).toDouble()) * 10.0),
        child: widgetValue);
  }

  Widget _getMobileWidget() {
    return Column(children: [
      Padding(
          padding: EdgeInsets.only(right: 10),
          child: SearchBox(
            searchTextController: _controller.searchController,
            placeholder: trans('Search'),
            onChange: (v) => _controller.globalSearchText(),
            onSubmit: (v) => _controller.globalSearchTextServer(),
          )),
      SizedBox(height: 20),
      Expanded(
          child: AnimationLimiter(
              child: SmartRefresher(
                  enablePullDown: true,
                  enablePullUp: true,
                  onRefresh: _controller.loadFromServer,
                  controller: _controller.refreshController,
                  header: WaterDropHeader(complete: Icon(Icons.done)),
                  footer: CustomFooter(
                    builder: (BuildContext context, LoadStatus? mode) {
                      Widget body;
                      if (mode == LoadStatus.idle) {
                        body = TecfyText("pull up load");
                      } else if (mode == LoadStatus.loading) {
                        body = CupertinoActivityIndicator();
                      } else if (mode == LoadStatus.failed) {
                        body = TecfyText("Load Failed!Click retry!");
                      } else if (mode == LoadStatus.canLoading) {
                        body = TecfyText("release to load more");
                      } else {
                        body = TecfyText("No more Data");
                      }
                      return Container(
                        height: 55.0,
                        child: Center(child: body),
                      );
                    },
                  ),
                  child: ListView.builder(
                      itemCount: _controller.filterdData?.length ?? 0,
                      itemBuilder: (BuildContext context, int index) {
                        return AnimationConfiguration.staggeredList(
                          position: index,
                          duration: const Duration(milliseconds: 375),
                          child: SlideAnimation(
                            verticalOffset: 50.0,
                            horizontalOffset: 50.5,
                            child: FlipAnimation(
                              flipAxis: FlipAxis.y,
                              child: _getMobileRow(
                                  _controller.filterdData![index]),
                            ),
                          ),
                        );
                      }))
              // ListView(
              //     // controller: _controller.scrollController,
              //     children:
              //         _controller.data.map<Widget>((d) => _getMobileRow(d)).toList()
              //     //..add(_controller.hasNextPage ? FlatButton
              //     (child:TecfyText
              //     ('Load More'),onPressed: _controller.gotoMextPageIndex) : Container())
              //     ),
              ))
    ]);
  }

  Widget _getMobileRow(d) {
    return Slidable(
      // delegate: SlidableDrawerDelegate(),
      startActionPane: ActionPane(
        motion: ScrollMotion(),
        children: _controller
            .getActions(d)
            .map<Widget>((a) => SlidableAction(
                  // An action can be bigger than the others.
                  flex: 2,
                  onPressed: (c) => _controller.onAction(a, d),
                  backgroundColor: ThemeService.mainColor,
                  foregroundColor: Colors.white,
                  icon: a.iconData,
                  label: a.name,
                ))
            .toList(),
      ),
      // actionExtentRatio: 0.25,
      child: Container(
          child: EntityListTileComponent(
        _controller.entity,
        d,
        showUpdateButton: false,
        onClick: (_d) => !widget.isListOnly &&
                _controller.entity.permission.canUpdate &&
                _controller.entity.entityType != EntityTypes.ViewOnly &&
                _controller.entity.entityType != EntityTypes.Lookup
            ? _controller.onUpdate(d)
            : _controller.onView(d),
        onExpandItem: (_d) => _controller.expandItem(_d),
      )),

      endActionPane: !_controller.entity.permission.canDelete ||
              widget.isListOnly ||
              _controller.entity.entityType == EntityTypes.ViewOnly ||
              _controller.entity.entityType == EntityTypes.Lookup
          ? null
          : ActionPane(
              motion: ScrollMotion(),
              children: [
                SlidableAction(
                  // An action can be bigger than the others.
                  flex: 2,
                  onPressed: (c) => _controller.onDelete(d),
                  backgroundColor: ThemeService.dangerColor,
                  foregroundColor: Colors.white,
                  icon: Icons.delete,
                  label: AppLocale.translate('Delete'),
                ),
              ],
            ),
    );
  }
}
